class ParsingModels:
  EN_DEFAULT = "sp_en_core_web_trf"
  SPACY_EN_CORE_WEB_TRF = "sp_en_core_web_trf"

class ParsingResultGranularity:
  SENTENCE = "sentence"
  TOKEN = "token"
  FULL_PARSE = "full_parse"
