class BlockTypes:
  Document = "doc"
  Page = "page" # E.g. in a PDF
  Region = "region" # E.g., abstract catchall region in a document
  Header = "header"
  H1 = "h1"
  H2 = "h2"
  H3 = "h3"
  H4 = "h4"
  H5 = "h5"
  Line = "line"
  Title = "title"
  Subtitle = "subtitle"
  Footer = "footer"
  Paragraph = "paragraph"
  List = "list"
  Blockquote = "blockquote"
  Blockcode = "blockcode"
  Unk = "unk"
  Sentence = "sentence"
  Token = "token"
  Span = "span"
