class FileFormats:
  UNKNOWN = "unknown"
  TXT = "txt"
  MKD = "mkd"
  EPUB = "epub"
  PDF = "pdf"
  JPG = "jpg"
  PNG = "png"
  TIFF = "tiff"
  GIF = "gif"
  HTML = "html"
  DOC = "doc"
  DOCX = "docx"
  PPT = "ppt"
  PPTX = "pptx"
  RTF = "rtf"
